package models

import (
	"eta/eta_index_lib/utils"
	"fmt"
	"strconv"
	"strings"
	"time"

	"github.com/beego/beego/v2/client/orm"
)

// BaseFromYongyi 涌溢咨询
type BaseFromYongyi struct{}

type BaseFromYongyiData struct {
	YongyiDataId  int `orm:"column(yongyi_data_id);pk"`
	YongyiIndexId int
	IndexCode     string
	DataTime      string
	Value         string
	CreateTime    time.Time
	ModifyTime    time.Time
	DataTimestamp int64
}

func GetBaseFromYongyiDataByCondition(condition string, pars []interface{}) (list []*BaseFromYongyiData, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM base_from_yongyi_data WHERE 1=1 `
	if condition != "" {
		sql += condition
	}
	_, err = o.Raw(sql, pars).QueryRows(&list)
	return
}

// Add 添加
func (obj BaseFromYongyi) Add(edbCode string) (err error) {
	o := orm.NewOrm()

	var condition string
	var pars []interface{}
	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}
	yongyiBaseDataAll, err := GetBaseFromYongyiDataByCondition(condition, pars)
	if err != nil && err.Error() != utils.ErrNoRow() {
		return
	}
	var isAdd bool
	addSql := ` INSERT INTO edb_data_yongyi(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	existMap := make(map[string]string)
	for _, sv := range yongyiBaseDataAll {
		eDate := sv.DataTime
		dataTime, err := time.Parse(utils.FormatDate, eDate)
		if err != nil {
			return err
		}
		timestamp := dataTime.UnixNano() / 1e6
		timeStr := fmt.Sprintf("%d", timestamp)
		if _, ok := existMap[eDate]; !ok {
			addSql += GetAddSql("0", edbCode, eDate, timeStr, sv.Value)
			isAdd = true
		}
		existMap[eDate] = sv.Value
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		utils.FileLog.Info("addSql:" + addSql)
		_, err = o.Raw(addSql).Exec()
		if err != nil {
			return err
		}
	}
	return
}

// Refresh 刷新涌益咨询指标数据
func (obj BaseFromYongyi) Refresh(edbInfoId int, edbCode, startDate string) (err error) {
	source := obj.GetSource()
	o := orm.NewOrm()
	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据
	var condition string
	var pars []interface{}

	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}

	yongyiDataList, err := GetBaseFromYongyiDataByCondition(condition, pars)
	if err != nil {
		return
	}

	// 真实数据的最大日期  ， 插入规则配置的日期
	var realDataMaxDate, edbDataInsertConfigDate time.Time
	var edbDataInsertConfig *EdbDataInsertConfig
	var isFindConfigDateRealData bool //是否找到配置日期的实际数据的值
	{
		edbDataInsertConfig, err = GetEdbDataInsertConfigByEdbId(edbInfoId)
		if err != nil && err.Error() != utils.ErrNoRow() {
			return
		}
		if edbDataInsertConfig != nil {
			edbDataInsertConfigDate = edbDataInsertConfig.Date
		}
	}

	var existCondition string
	var existPars []interface{}

	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)
	if startDate != "" {
		existCondition += " AND data_time>=? "
		existPars = append(existPars, startDate)
	}

	existList, err := GetEdbDataByCondition(source, 0, existCondition, existPars)
	if err != nil {
		return err
	}
	existMap := make(map[string]*EdbInfoSearchData)
	for _, v := range existList {
		existMap[v.DataTime] = v
	}
	addSql := ` INSERT INTO edb_data_yongyi(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	var isAdd bool
	for _, v := range yongyiDataList {
		item := v
		eDate := item.DataTime
		dataTime, err := time.ParseInLocation(utils.FormatDate, eDate, time.Local)
		if err != nil {
			return err
		}
		if findItem, ok := existMap[v.DataTime]; !ok {
			sValue := item.Value

			timestamp := dataTime.UnixNano() / 1e6
			timeStr := fmt.Sprintf("%d", timestamp)

			addSql += GetAddSql(edbInfoIdStr, edbCode, eDate, timeStr, sValue)
			isAdd = true
		} else {
			if findItem != nil && utils.SubFloatToString(findItem.Value, 30) != item.Value {
				err = ModifyEdbDataById(source, 0, findItem.EdbDataId, item.Value)
				if err != nil {
					return err
				}
			}
		}

		// 下面代码主要目的是处理掉手动插入的数据判断
		{
			if realDataMaxDate.IsZero() || dataTime.After(realDataMaxDate) {
				realDataMaxDate = dataTime
			}
			if edbDataInsertConfigDate.IsZero() || dataTime.Equal(edbDataInsertConfigDate) {
				isFindConfigDateRealData = true
			}
		}
	}

	// 处理手工数据补充的配置
	HandleConfigInsertEdbData(realDataMaxDate, edbDataInsertConfig, edbInfoId, source, 0, existMap, isFindConfigDateRealData)

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = o.Raw(addSql).Exec()
		if err != nil {
			fmt.Println("RefreshEdbDataFromBaiinfo add Err", err.Error())
			return
		}
	}
	return
}

// GetSource 获取来源编码id
func (obj BaseFromYongyi) GetSource() int {
	return utils.DATA_SOURCE_YONYI
}

// GetSourceName 获取来源名称
func (obj BaseFromYongyi) GetSourceName() string {
	return utils.DATA_SOURCE_NAME_YONYI
}

type BaseFromYongyiIndex struct {
	YongyiIndexId int64 `orm:"column(yongyi_index_id);pk"`
	IndexCode     string
	IndexName     string
	Frequency     string
	Unit          string
	ClassifyId    int64
	StartDate     string
	EndDate       string
	Sort          int
	TerminalCode  string
	CreateTime    time.Time
	ModifyTime    time.Time
}

type BaseFromYongyiIndexList struct {
	YongyiIndexId int64 `orm:"column(yongyi_index_id);pk"`
	IndexCode     string
	IndexName     string
	Frequency     string
	Unit          string
	ClassifyId    int64
	StartDate     string
	EndDate       string
	TerminalCode  string
	CreateTime    string
	ModifyTime    string
}

func (y *BaseFromYongyiData) GetByIndexCode(indexCode string) (list []*BaseFromYongyiData, err error) {
	o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_yongyi_data WHERE index_code=? `
	_, err = o.Raw(sql, indexCode).QueryRows(&list)
	return
}

func (y *BaseFromYongyiData) AddMulti(item []*BaseFromYongyiData) (err error) {
	o := orm.NewOrm()
	_, err = o.InsertMulti(1, item)
	return
}

// Update 修改
func (y *BaseFromYongyiData) Update(updateCols []string) (err error) {
	o := orm.NewOrm()
	_, err = o.Update(y, updateCols...)
	return
}

// HandleYongyiExcelData 涌益咨询的excel数据
type HandleYongyiExcelData struct {
	ClassifyName string `description:"指标目录"`
	ClassifySort int    `description:"指标目录排序号"`
	IndexName    string `description:"指标名称"`
	IndexCode    string `description:"指标编码"`
	Unit         string `description:"单位"`
	Sort         int    `description:"排序号"`
	Frequency    string `description:"频度"`
	ExcelDataMap map[string]string
}

type HandleYongyiExcelDataReq struct {
	List         []*HandleYongyiExcelData
	TerminalCode string `description:"编码"`
}

func (y *BaseFromYongyiData) GetMaxAndMinDateByIndexCode(indexCode string) (item *EdbInfoMaxAndMinInfo, err error) {
	o := orm.NewOrm()
	sql := ` SELECT MIN(data_time) AS min_date,MAX(data_time) AS max_date,MIN(value) AS min_value,MAX(value) AS max_value FROM base_from_yongyi_data WHERE index_code=? `
	err = o.Raw(sql, indexCode).QueryRow(&item)
	return
}

func (y *BaseFromYongyiIndex) ModifyIndexMaxAndMinDate(indexCode string, item *EdbInfoMaxAndMinInfo) (err error) {
	o := orm.NewOrm()
	sql := ` UPDATE base_from_yongyi_index SET start_date=?,end_date=?,modify_time=NOW() WHERE index_code=? `
	_, err = o.Raw(sql, item.MinDate, item.MaxDate, indexCode).Exec()
	return
}

func (y *BaseFromYongyiIndex) GetByIndexCode(indexCode string) (item *BaseFromYongyiIndex, err error) {
	o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_yongyi_index WHERE index_code=? `
	err = o.Raw(sql, indexCode).QueryRow(&item)
	return
}

func (y *BaseFromYongyiIndex) Add() (lastId int64, err error) {
	o := orm.NewOrm()
	lastId, err = o.Insert(y)
	return
}

// Update 修改
func (y *BaseFromYongyiIndex) Update(updateCols []string) (err error) {
	o := orm.NewOrm()
	_, err = o.Update(y, updateCols...)

	return
}

// BaseFromYongyiClassify Yongyi原始数据分类表
type BaseFromYongyiClassify struct {
	ClassifyId      int64     `orm:"column(classify_id);pk"`
	ClassifyName    string    `description:"分类名称"`
	ParentId        int       `description:"父级id"`
	SysUserId       int       `description:"创建人id"`
	SysUserRealName string    `description:"创建人姓名"`
	Level           int       `description:"层级"`
	Sort            int       `description:"排序字段，越小越靠前，默认值：10"`
	ModifyTime      time.Time `description:"修改时间"`
	CreateTime      time.Time `description:"创建时间"`
}

func (y *BaseFromYongyiClassify) Add() (lastId int64, err error) {
	o := orm.NewOrm()
	lastId, err = o.Insert(y)
	return
}

// Update 修改
func (y *BaseFromYongyiClassify) Update(updateCols []string) (err error) {
	o := orm.NewOrm()
	_, err = o.Update(y, updateCols...)

	return
}

func (y *BaseFromYongyiClassify) GetByClassifyName(classifyName string) (item *BaseFromYongyiClassify, err error) {
	o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_yongyi_classify WHERE classify_name=? `
	err = o.Raw(sql, classifyName).QueryRow(&item)
	return
}

func (y *BaseFromYongyiClassify) GetParentClassify() (items []*BaseFromYongyiClassify, err error) {
	o := orm.NewOrm()
	sql := ` SELECT * FROM base_from_yongyi_classify WHERE classify_name in ("日度", "月度", "周度", "年度", "旬度") and parent_id=0 `
	_, err = o.Raw(sql).QueryRows(&items)
	return
}
